SystemOrganization addCategory: #KSRSummarizer!

Morph subclass: #KSRSummarizer
	instanceVariableNames: 'ui database databaseValues minimums maximums columnCount converters subjects quantifiers attributes summaries count'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KSRSummarizer'!

!KSRSummarizer methodsFor: 'database' stamp: 'mg 6/30/2014 09:31'!
convertRow: row
	|point first|
	first := true.
	row with: converters do: [:value :converter|
		first 
		ifTrue: 
		[
			point := converter convertValue: value.
			first := false
		] 
		ifFalse: 
		[
			point := point @@ (converter convertValue: value)
		]
	].
	^ point! !

!KSRSummarizer methodsFor: 'accessing' stamp: 'mg 6/30/2014 01:22'!
converters
	^ converters! !

!KSRSummarizer methodsFor: 'accessing' stamp: 'mg 6/30/2014 01:22'!
converters: aCollection
	converters := aCollection! !

!KSRSummarizer methodsFor: 'database' stamp: 'mg 6/30/2014 09:43'!
database
	^ database! !

!KSRSummarizer methodsFor: 'database' stamp: 'mg 7/2/2014 00:21'!
findDatabaseValues
	databaseValues := Array new: columnCount.
	1 to: columnCount do: [:index| databaseValues at: index put: Set new].
	
	database do:
	[:row|
		|point|
		point := row.
		columnCount to: 2 by: -1 do: 
		[:index|
			(databaseValues at: index) add: point y.
			point := point x.
		].
		(databaseValues at: 1) add: point.
	]
! !

!KSRSummarizer methodsFor: 'database' stamp: 'mg 7/2/2014 00:24'!
findMaximums
	maximums := databaseValues collect:
	[:set|
		set anyOne isNumber 
		ifTrue: [set max]
		ifFalse: [nil]
	]! !

!KSRSummarizer methodsFor: 'database' stamp: 'mg 7/2/2014 00:24'!
findMinimums
	minimums := databaseValues collect:
	[:set|
		set anyOne isNumber 
		ifTrue: [set min]
		ifFalse: [nil]
	]! !

!KSRSummarizer methodsFor: 'initialize-release' stamp: 'mg 6/30/2014 14:40'!
initialize
	super initialize.
	converters :=
	{
		KSRNumberConverter. "age"
		KSRSymbolConverter. "job"
		KSRSymbolConverter. "marital"
		KSRSymbolConverter. "education"
		KSRSymbolConverter. "default"
		KSRSymbolConverter. "housing"
		KSRSymbolConverter. "loan"
		KSRSymbolConverter. "contact"
		KSRSymbolConverter. "month"
		KSRSymbolConverter. "day_of_week"
		KSRNumberConverter. "duration"
		KSRNumberConverter. "campaign"
		KSRNumberConverter. "pdays"
		KSRNumberConverter. "previous"
		KSRSymbolConverter. "poutcome"
		KSRNumberConverter. "emp.var.rate"
		KSRNumberConverter. "cons.price.idx"
		KSRNumberConverter. "cons.conf.idx"
		KSRNumberConverter. "euribor3m"
		KSRNumberConverter. "nr.employed"
		KSRBooleanConverter. "deposit"
	}.
	summaries := SortedCollection sortBlock: [:a :b| a >= b].
	count := 1.
	self ui setupUi: self.
	"self ui repeatitions theText: '1'."
	self connect: self ui gen signal: #clicked: to: self selector: #run.
	self connect: self ui save signal: #clicked: to: self selector: #save.	
	self connect: self ui load signal: #clicked: to: self selector: #load.			
	self openInWindowLabeled: 'Fuzzy Summarizer'.	! !

!KSRSummarizer methodsFor: 'loading' stamp: 'mg 6/30/2014 15:30'!
load
	|stream|
	[
		"stream := FileChooser new initializeAsSystemWindow; open."
		stream := FileStream fileNamed: '/home/mateusz/ksr/dane/bank-additional/bank-additional.csv'.
		stream ifNil: [^ self].
		self loadDatabaseFrom: stream.
	] ensure: 
	[
		stream ifNotNil: [stream close]
	]! !

!KSRSummarizer methodsFor: 'database' stamp: 'mg 7/2/2014 00:14'!
loadDatabaseFrom: aStream
	|isFirstRow|
	isFirstRow := true.
	database := OrderedCollection new.
	(CSVParser on: aStream) useDelimiter: $;; rowsDo:
	[:row|
		isFirstRow 
			ifTrue: [isFirstRow := false. columnCount := row size]
			ifFalse: [database add: (self convertRow: row)]
	].

	self findDatabaseValues.
	self findMaximums.
	self findMinimums.
	! !

!KSRSummarizer methodsFor: 'database' stamp: 'mg 7/2/2014 00:06'!
maximums
	^ maximums! !

!KSRSummarizer methodsFor: 'database' stamp: 'mg 7/2/2014 00:06'!
minimums
	^ minimums! !

!KSRSummarizer methodsFor: 'generation' stamp: 'mg 7/2/2014 02:15'!
run
	|summary goodness list summaryBlock|
	
	summaryBlock := [:cls|
		summary := cls random.
		goodness := summary 
			summarize: database
			values: databaseValues
			minimums: minimums
			maximums: maximums.
		summaries add: goodness -> summary.
	].
	
	summaries removeAll.
	ui summaries theText: ''.
	list := ''.
	count := ui repeatitions theText asNumber.
	
	1 to: count do:
	[:i|
		summaryBlock value: KSRYagerAbsoluteSummary.				
		summaryBlock value: KSRYagerRelativeSummary.
		summaryBlock value: KSRFirstMultiSubjectSummary.				
		summaryBlock value: KSRSecondMultiSubjectSummary.
		summaryBlock value: KSRThirdMultiSubjectSummary.														
		summaryBlock value: KSRMoreSummary.
	].

	summaries do: 
	[:s|
		list := 
			list,
			'[',
			s key,
			'] ',
			s value description,
			' T1:',
			s value t1,
			' T2:',
			s value t2,
			String cr
	].

	ui summaries theText: list.! !

!KSRSummarizer methodsFor: 'saving' stamp: 'mg 6/30/2014 14:46'!
save
	|stream|
	[
		stream := FileStream fileNamed: 'summaries.txt'.
		stream nextPutAll: (ui summaries theText).
		Transcript showln: 'Saved!!'
	] ensure:
	[
		stream ifNotNil: [stream close].
	]! !

!KSRSummarizer methodsFor: 'accessing' stamp: 'mg 6/30/2014 14:07'!
ui
	^ ui ifNil: [ui := KSRSummarizerUi new].! !

UserInterface subclass: #KSRSummarizerUi
	instanceVariableNames: 'save gen load repeatitions summaries'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KSRSummarizer'!

!KSRSummarizerUi methodsFor: 'accessing' stamp: 'mg 6/30/2014 14:36'!
gen
	"No description given."
	
	^ gen! !

!KSRSummarizerUi methodsFor: 'ui - private' stamp: 'mg 6/30/2014 14:36'!
initializePanel
	"This code is auto-generated. DO NOT modify it because another generation step will overwrite all changes."
	
	|  |
	
	panel := UiContainer new.
	panel extent: (600@500).
	panel position: (0@0).
	panel hResizing: (#rigid).
	panel vResizing: (#rigid).
	panel minWidth: (2).
	panel minHeight: (2).
	panel cellInset: (4).
	panel cellPositioning: (#center).
	panel cellSpacing: (#none).
	panel layoutInset: (4).
	panel listCentering: (#topLeft).
	panel listDirection: (#topToBottom).
	panel listSpacing: (#none).
	panel maxCellSize: (1073741823).
	panel minCellSize: (0).
	panel wrapCentering: (#topLeft).
	panel wrapDirection: (#none).
	panel frameFractions: (0@0 corner: 1@1).
	panel frameOffsets: (0@0 corner: 0@0).
	panel balloonText: (nil).
	panel balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.722)).
	panel color: (Color veryLightGray).
	panel borderWidth: (0).
	panel borderStyle2: (#simple).
	panel borderColor: (Color black).
	panel setProperty: #objectName toValue: 'panel'.
	panel setProperty: #objectDescription toValue: 'Interfejs użytkownika dla sumaryzatora'.
	save := UiPushButton new.
	save changeTableLayout.
	save extent: (31@18).
	save position: (231@14).
	save hResizing: (#shrinkWrap).
	save vResizing: (#shrinkWrap).
	save minWidth: (2).
	save minHeight: (2).
	save cellInset: (4).
	save cellPositioning: (#center).
	save cellSpacing: (#none).
	save layoutInset: (2@0 corner: 2@0).
	save listCentering: (#topLeft).
	save listDirection: (#leftToRight).
	save listSpacing: (#none).
	save maxCellSize: (1073741823).
	save minCellSize: (0).
	save wrapCentering: (#topLeft).
	save wrapDirection: (#none).
	save frameFractions: (0@0 corner: 1@1).
	save frameOffsets: (0@0 corner: 0@0).
	save balloonText: (nil).
	save balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.745)).
	save enabled: (true).
	save setProperty: #iconByDesigner toValue: 'nil'.
	save icon: (nil).
	save text: ('Save').
	save color: ((Color r: 0.8 g: 0.8 b: 0.8)).
	save checkable: (false).
	save checked: (false).
	save autoExclusive: (false).
	save setProperty: #objectName toValue: 'save'.
	panel addChild: save.
	gen := UiPushButton new.
	gen changeTableLayout.
	gen extent: (53@18).
	gen position: (171@14).
	gen hResizing: (#shrinkWrap).
	gen vResizing: (#shrinkWrap).
	gen minWidth: (2).
	gen minHeight: (2).
	gen cellInset: (4).
	gen cellPositioning: (#center).
	gen cellSpacing: (#none).
	gen layoutInset: (2@0 corner: 2@0).
	gen listCentering: (#topLeft).
	gen listDirection: (#leftToRight).
	gen listSpacing: (#none).
	gen maxCellSize: (1073741823).
	gen minCellSize: (0).
	gen wrapCentering: (#topLeft).
	gen wrapDirection: (#none).
	gen frameFractions: (0@0 corner: 1@1).
	gen frameOffsets: (0@0 corner: 0@0).
	gen balloonText: (nil).
	gen balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.733)).
	gen enabled: (true).
	gen setProperty: #iconByDesigner toValue: 'nil'.
	gen icon: (nil).
	gen text: ('Generate').
	gen color: ((Color r: 0.8 g: 0.8 b: 0.8)).
	gen checkable: (false).
	gen checked: (false).
	gen autoExclusive: (false).
	gen setProperty: #objectName toValue: 'gen'.
	panel addChild: gen.
	load := UiPushButton new.
	load changeTableLayout.
	load extent: (81@18).
	load position: (83@14).
	load hResizing: (#shrinkWrap).
	load vResizing: (#shrinkWrap).
	load minWidth: (2).
	load minHeight: (2).
	load cellInset: (4).
	load cellPositioning: (#center).
	load cellSpacing: (#none).
	load layoutInset: (2@0 corner: 2@0).
	load listCentering: (#topLeft).
	load listDirection: (#leftToRight).
	load listSpacing: (#none).
	load maxCellSize: (1073741823).
	load minCellSize: (0).
	load wrapCentering: (#topLeft).
	load wrapDirection: (#none).
	load frameFractions: (0@0 corner: 1@1).
	load frameOffsets: (0@0 corner: 0@0).
	load balloonText: (nil).
	load balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.749)).
	load enabled: (true).
	load setProperty: #iconByDesigner toValue: 'nil'.
	load icon: (nil).
	load text: ('Load database').
	load color: ((Color r: 0.8 g: 0.8 b: 0.8)).
	load checkable: (false).
	load checked: (false).
	load autoExclusive: (false).
	load setProperty: #objectName toValue: 'load'.
	panel addChild: load.
	repeatitions := UiLineEdit new.
	repeatitions extent: (41@17).
	repeatitions position: (37@16).
	repeatitions hResizing: (#spaceFill).
	repeatitions vResizing: (#rigid).
	repeatitions minWidth: (2).
	repeatitions minHeight: (2).
	repeatitions cellInset: (0).
	repeatitions cellPositioning: (#center).
	repeatitions cellSpacing: (#none).
	repeatitions layoutInset: (0).
	repeatitions listCentering: (#topLeft).
	repeatitions listDirection: (#topToBottom).
	repeatitions listSpacing: (#none).
	repeatitions maxCellSize: (1073741823).
	repeatitions minCellSize: (0).
	repeatitions wrapCentering: (#topLeft).
	repeatitions wrapDirection: (#none).
	repeatitions frameFractions: (0@0 corner: 1@1).
	repeatitions frameOffsets: (0@0 corner: 0@0).
	repeatitions balloonText: (nil).
	repeatitions balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.737)).
	repeatitions widgetResizable: (false).
	repeatitions horizontalScrollBarPolicy: (#alwaysOff).
	repeatitions verticalScrollBarPolicy: (#alwaysOff).
	repeatitions outerBorderWidth: (1).
	repeatitions scrollBarThickness: (9).
	repeatitions theText: ('10').
	repeatitions readOnly: (false).
	repeatitions autoConvert: (false).
	repeatitions autoAccept: (false).
	repeatitions acceptOnFocusLost: (true).
	repeatitions scrollingMode: (#none).
	repeatitions scrollingLimit: (10).
	repeatitions helpText: ('').
	repeatitions hideCursor: (true).
	repeatitions setProperty: #objectName toValue: 'repeatitions'.
	panel addChild: repeatitions.
	summaries := UiTextEdit new.
	summaries extent: (523@415).
	summaries position: (38@62).
	summaries hResizing: (#rigid).
	summaries vResizing: (#rigid).
	summaries minWidth: (2).
	summaries minHeight: (2).
	summaries cellInset: (0).
	summaries cellPositioning: (#center).
	summaries cellSpacing: (#none).
	summaries layoutInset: (0).
	summaries listCentering: (#topLeft).
	summaries listDirection: (#topToBottom).
	summaries listSpacing: (#none).
	summaries maxCellSize: (1073741823).
	summaries minCellSize: (0).
	summaries wrapCentering: (#topLeft).
	summaries wrapDirection: (#none).
	summaries frameFractions: (0@0 corner: 1@1).
	summaries frameOffsets: (0@0 corner: 0@0).
	summaries balloonText: (nil).
	summaries balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.729)).
	summaries widgetResizable: (true).
	summaries horizontalScrollBarPolicy: (#alwaysOff).
	summaries verticalScrollBarPolicy: (#alwaysOn).
	summaries outerBorderWidth: (1).
	summaries scrollBarThickness: (9).
	summaries theText: ('').
	summaries readOnly: (false).
	summaries autoConvert: (false).
	summaries autoAccept: (false).
	summaries acceptOnFocusLost: (false).
	summaries scrollingMode: (#none).
	summaries scrollingLimit: (10).
	summaries setProperty: #objectName toValue: 'summaries'.
	panel addChild: summaries.
	panel setProperty: #uiClassName toValue: 'KSRSummarizer'.! !

!KSRSummarizerUi methodsFor: 'accessing' stamp: 'mg 6/30/2014 14:36'!
load
	"No description given."
	
	^ load! !

!KSRSummarizerUi methodsFor: 'accessing' stamp: 'mg 6/30/2014 14:36'!
repeatitions
	"No description given."
	
	^ repeatitions! !

!KSRSummarizerUi methodsFor: 'accessing' stamp: 'mg 6/30/2014 14:36'!
save
	"No description given."
	
	^ save! !

!KSRSummarizerUi methodsFor: 'accessing' stamp: 'mg 6/30/2014 14:36'!
summaries
	"No description given."
	
	^ summaries! !

Point subclass: #KSRPoint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KSRSummarizer'!

!KSRPoint methodsFor: 'printing' stamp: 'mg 6/29/2014 20:34'!
printOn: aStream
	aStream print: x.
	aStream nextPutAll: ' @@ '.
	aStream print: y! !

Object subclass: #KSRConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KSRSummarizer'!

KSRConverter subclass: #KSRBooleanConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KSRSummarizer'!

!KSRBooleanConverter class methodsFor: 'as yet unclassified' stamp: 'mg 6/30/2014 01:30'!
convertValue: value
	value = 'yes' ifTrue: [^true] ifFalse: [^false]! !

!KSRConverter class methodsFor: 'converting' stamp: 'mg 6/30/2014 01:25'!
convertValue: value
	self subclassResponsibility! !

!KSRConverter methodsFor: 'converting' stamp: 'mg 6/30/2014 01:19'!
convertValue: aValue
	self subclassResponsibility! !

KSRConverter subclass: #KSRNumberConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KSRSummarizer'!

!KSRNumberConverter class methodsFor: 'converting' stamp: 'mg 6/30/2014 01:25'!
convertValue: value
	^ value asNumber! !

KSRConverter subclass: #KSRStringConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KSRSummarizer'!

!KSRStringConverter class methodsFor: 'converting' stamp: 'mg 6/30/2014 01:26'!
convertValue: value
	^ value asString! !

KSRConverter subclass: #KSRSymbolConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KSRSummarizer'!

!KSRSymbolConverter class methodsFor: 'as yet unclassified' stamp: 'mg 6/30/2014 01:40'!
convertValue: aValue
	^ aValue asSymbol! !

Object subclass: #KSRFuzzySet
	instanceVariableNames: 'name membership specials height constituents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KSRSummarizer'!

!KSRFuzzySet class methodsFor: 'simple sets' stamp: 'mg 6/29/2014 23:55'!
absent
	^ self new membership: [:a|0]; name: 'absent'! !

!KSRFuzzySet class methodsFor: 'magnitude sets' stamp: 'mg 6/30/2014 01:59'!
equalOrGreaterThan: anObject
	^ self new
		membership: [:a| a >= anObject ifTrue: [1] ifFalse: [0]];
		specials: (Set with: anObject);
		name: 'equal or greater than ',anObject asString! !

!KSRFuzzySet class methodsFor: 'magnitude sets' stamp: 'mg 6/30/2014 01:59'!
equalOrLessThan: anObject
	^ self new
		membership: [:a| a <= anObject ifTrue: [1] ifFalse: [0]];
		specials: (Set with: anObject);
		name: 'equal or less than ',anObject asString		! !

!KSRFuzzySet class methodsFor: 'magnitude sets' stamp: 'mg 6/30/2014 01:59'!
greaterThan: anObject
	^ self new
		membership: [:a| a > anObject ifTrue: [1] ifFalse: [0]];
		specials: (Set with: anObject);
		name: 'greater than ',anObject asString		! !

!KSRFuzzySet class methodsFor: 'magnitude sets' stamp: 'mg 6/30/2014 01:59'!
lessThan: anObject
	^ self new
		membership: [:a| a < anObject ifTrue: [1] ifFalse: [0]];
		specials: (Set with: anObject);
		name: 'less than ',anObject asString		! !

!KSRFuzzySet class methodsFor: 'simple sets' stamp: 'mg 6/29/2014 23:56'!
present
	^ self new membership: [:a|1]; name: 'present'! !

!KSRFuzzySet class methodsFor: 'specials' stamp: 'mg 6/30/2014 03:25'!
specialsLimit
	^ 10000! !

!KSRFuzzySet class methodsFor: 'simple sets' stamp: 'mg 6/30/2014 02:26'!
withElements: aCollection
	|set dictionary|
	set := Set new.
	dictionary := Dictionary new.
	aCollection do:
	[:o|
		set add: o key.
		dictionary at: o key put: o value
	].

	^ self new
		membership: [:a| dictionary at: a ifAbsent: [0]];
		specials: set;
		name: 'in ',aCollection printString! !

!KSRFuzzySet methodsFor: 'operations' stamp: 'mg 6/30/2014 02:30'!
* aNumber
	^ self class new 
		membership: [:a|
			self ? a * aNumber clampLow: 0 high: 1		
		];
		specials: specials;
		name: name,'*',aNumber asString! !

!KSRFuzzySet methodsFor: 'operations' stamp: 'mg 6/30/2014 02:31'!
+ aNumber
	^ self class new 
		membership: [:a|
			self ? a + aNumber clampLow: 0 high: 1		
		];
		specials: specials;
		name: name,'+',aNumber asString		! !

!KSRFuzzySet methodsFor: 'operations' stamp: 'mg 6/30/2014 02:31'!
- aNumber
	^ self class new 
		membership: [:a|
			self ? a - aNumber clampLow: 0 high: 1		
		];
		specials: specials;
		name: name,'-',aNumber asString		! !

!KSRFuzzySet methodsFor: 'operations' stamp: 'mg 6/30/2014 02:31'!
/ aNumber
	^ self class new 
		membership: [:a|
			self ? a / aNumber clampLow: 0 high: 1		
		];
		specials: specials;
		name: name,'/',aNumber asString		! !

!KSRFuzzySet methodsFor: 'operations' stamp: 'mg 6/30/2014 03:21'!
/\ aKSRFuzzySet
	^ self and: aKSRFuzzySet norm: [:a :b| a min: b]! !

!KSRFuzzySet methodsFor: 'operations' stamp: 'mg 7/2/2014 01:39'!
<> aKSRFuzzySet
	^ self OR: aKSRFuzzySet norm: [:a :b| a max: b]! !

!KSRFuzzySet methodsFor: 'operations' stamp: 'mg 6/30/2014 03:24'!
>< aKSRFuzzySet
	^ self AND: aKSRFuzzySet norm: [:a :b| a min: b]! !

!KSRFuzzySet methodsFor: 'membership' stamp: 'mg 6/29/2014 20:43'!
? anObject
	^ membership value: anObject! !

!KSRFuzzySet methodsFor: 'operations' stamp: 'mg 7/1/2014 22:09'!
AND: aKSRFuzzySet norm: norm
	|newSpecials|
	
	newSpecials := Set new.
	
	specials do: [:s1|
		aKSRFuzzySet specials do: [:s2|
			newSpecials add: s1 @@ s2
		]
	].

	"Gdy jest ich za dużo to przestaję liczyć."
	newSpecials size > self class specialsLimit ifTrue: [newSpecials := Set new].
	
	^ self class new 
		membership: [:object| norm value: self ? object x value: aKSRFuzzySet ? object y];
		specials: newSpecials;
		constituents: {self. aKSRFuzzySet};
		name: self printString,' AND ',aKSRFuzzySet printString! !

!KSRFuzzySet methodsFor: 'operations' stamp: 'mg 7/2/2014 01:39'!
OR: aKSRFuzzySet norm: norm
	|newSpecials|
	
	newSpecials := Set new.
	
	specials do: [:s1|
		aKSRFuzzySet specials do: [:s2|
			newSpecials add: s1 @@ s2
		]
	].

	"Gdy jest ich za dużo to przestaję liczyć."
	newSpecials size > self class specialsLimit ifTrue: [newSpecials := Set new].
	
	^ self class new 
		membership: [:object| norm value: self ? object x value: aKSRFuzzySet ? object y];
		specials: newSpecials;
		constituents: {self. aKSRFuzzySet};
		name: self printString,' OR ',aKSRFuzzySet printString! !

!KSRFuzzySet methodsFor: 'operations' stamp: 'mg 6/30/2014 03:21'!
\/ aKSRFuzzySet
	^ self or: aKSRFuzzySet norm: [:a :b| a max: b]! !

!KSRFuzzySet methodsFor: 'accessing' stamp: 'mg 7/1/2014 22:10'!
addConstituentsTo: anOrderedCollection
	constituents = self
	ifTrue:
	[
		anOrderedCollection add: self
	]
	ifFalse:
	[
		constituents first addConstituentsTo: anOrderedCollection.
		constituents second addConstituentsTo: anOrderedCollection
	]! !

!KSRFuzzySet methodsFor: 'operations' stamp: 'mg 6/30/2014 02:47'!
alphaCut: aNumber
	^ self class new
		membership: [:o| self ? o >= aNumber ifTrue: [1] ifFalse: [0]];
		specials: specials;
		name: 'alpha cut at ',aNumber asString,' of ',self printString
		! !

!KSRFuzzySet methodsFor: 'operations' stamp: 'mg 6/30/2014 03:22'!
and: aKSRFuzzySet norm: aBlock
	^ self class new 
		membership: [:object| aBlock value: self ? object value: aKSRFuzzySet ? object];
		specials: specials, aKSRFuzzySet specials;
		name: self printString,' and ',aKSRFuzzySet printString	! !

!KSRFuzzySet methodsFor: 'operations' stamp: 'mg 7/2/2014 01:58'!
card
	^ (specials asArray collect: [:special| self ? special])
	ifEmpty: [^0]
	ifNotEmpty: [:values| values sum].! !

!KSRFuzzySet methodsFor: 'accessing' stamp: 'mg 7/1/2014 22:01'!
constituents

	^ constituents! !

!KSRFuzzySet methodsFor: 'accessing' stamp: 'mg 7/1/2014 22:01'!
constituents: anObject

	constituents := anObject! !

!KSRFuzzySet methodsFor: 'accessing' stamp: 'mg 7/2/2014 01:34'!
constituentsCollection
	|collection|
	collection := OrderedCollection new.
	self addConstituentsTo: collection.
	^ collection! !

!KSRFuzzySet methodsFor: 'operations' stamp: 'mg 7/2/2014 01:14'!
fuzzinessForDomainWithCardinality: aNumber
	^ self support card / aNumber! !

!KSRFuzzySet methodsFor: 'operations' stamp: 'mg 6/30/2014 03:04'!
height
	height ifNotNil: [^height].
	"Poniższe może się zdarzyć, w dwóch wypadkach:
		a) gdy nie ma elementów specjalnych
		b) gdy jest za dużo elementów specjalnych"
	specials ifEmpty: [^0].
	^ (specials collect: [:s| self ? s]) max! !

!KSRFuzzySet methodsFor: 'initialize-release' stamp: 'mg 7/1/2014 22:00'!
initialize
	super initialize.
	membership := [:x|0].
	specials := Set new.
	constituents := self! !

!KSRFuzzySet methodsFor: 'testing' stamp: 'mg 6/30/2014 10:35'!
isNumericFuzzySet
	^ false! !

!KSRFuzzySet methodsFor: 'operations' stamp: 'mg 6/29/2014 23:30'!
kernel
	^ self alphaCut: 1! !

!KSRFuzzySet methodsFor: 'membership' stamp: 'mg 6/29/2014 19:36'!
membership
	^ membership! !

!KSRFuzzySet methodsFor: 'membership' stamp: 'mg 6/29/2014 19:36'!
membership: aBlock
	membership := aBlock! !

!KSRFuzzySet methodsFor: 'accessing' stamp: 'mg 6/29/2014 21:30'!
name
	^ name! !

!KSRFuzzySet methodsFor: 'accessing' stamp: 'mg 6/29/2014 21:30'!
name: aString
	name := aString! !

!KSRFuzzySet methodsFor: 'operations' stamp: 'mg 6/30/2014 02:37'!
not
	^ self class new 
		membership: [:object| 1 - (self ? object)];
		specials: specials;
		name: 'not ',self printString		! !

!KSRFuzzySet methodsFor: 'operations' stamp: 'mg 6/30/2014 03:20'!
or: aKSRFuzzySet norm: aBlock
	^ self class new 
		membership: [:object| aBlock value: self ? object value: aKSRFuzzySet ? object];
		specials: specials, aKSRFuzzySet specials;
		name: self printString,' or ',aKSRFuzzySet printString			! !

!KSRFuzzySet methodsFor: 'printing' stamp: 'mg 6/29/2014 23:58'!
printOn: aStream
	^ name 
		ifNil: [^ super printOn: aStream]
		ifNotNil: [aStream nextPutAll: name]
	! !

!KSRFuzzySet methodsFor: 'accessing' stamp: 'mg 6/29/2014 21:13'!
specials
	^ specials! !

!KSRFuzzySet methodsFor: 'accessing' stamp: 'mg 6/29/2014 21:14'!
specials: aSet
	specials := aSet! !

!KSRFuzzySet methodsFor: 'operations' stamp: 'mg 6/30/2014 02:37'!
strictAlphaCut: aNumber
	^ self class new
		membership: [:o| self ? o > aNumber ifTrue: [1] ifFalse: [0]];
		specials: specials.
		! !

!KSRFuzzySet methodsFor: 'operations' stamp: 'mg 6/29/2014 23:31'!
support
	^ self strictAlphaCut: 0! !

KSRFuzzySet subclass: #KSRMagnitudeFuzzySet
	instanceVariableNames: 'comparator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KSRSummarizer'!

!KSRMagnitudeFuzzySet class methodsFor: 'magnitude sets' stamp: 'mg 6/30/2014 02:12'!
equalOrGreaterThan: anObject
	|s|
	^ (s:=self new)
		membership: [:a| (s element: a equalOrGreaterThan: anObject) ifTrue: [1] ifFalse: [0]];
		specials: (Set with: anObject);
		name: 'equal or greater than ',anObject asString! !

!KSRMagnitudeFuzzySet class methodsFor: 'magnitude sets' stamp: 'mg 6/30/2014 02:14'!
equalOrLessThan: anObject
	|s|
	^ (s:=self new)
		membership: [:a| (s element: a equalOrLessThan: anObject) ifTrue: [1] ifFalse: [0]];
		specials: (Set with: anObject);
		name: 'equal or less than ',anObject asString		! !

!KSRMagnitudeFuzzySet class methodsFor: 'magnitude sets' stamp: 'mg 6/30/2014 02:13'!
greaterThan: anObject
	|s|
	^ (s:=self new)
		membership: [:a| (s element: a greaterThan: anObject) ifTrue: [1] ifFalse: [0]];
		specials: (Set with: anObject);
		name: 'greater than ',anObject asString		! !

!KSRMagnitudeFuzzySet class methodsFor: 'magnitude sets' stamp: 'mg 6/30/2014 02:14'!
lessThan: anObject
	|s|
	^ (s:=self new)
		membership: [:a| (s element: a lessThan: anObject) ifTrue: [1] ifFalse: [0]];
		specials: (Set with: anObject);
		name: 'less than ',anObject asString		! !

!KSRMagnitudeFuzzySet methodsFor: 'element comparison' stamp: 'mg 6/30/2014 02:18'!
comparator
	^ comparator! !

!KSRMagnitudeFuzzySet methodsFor: 'element comparison' stamp: 'mg 6/30/2014 02:18'!
comparator: aBlock
	comparator := aBlock! !

!KSRMagnitudeFuzzySet methodsFor: 'element comparison' stamp: 'mg 6/30/2014 02:23'!
element: e1 equalOrGreaterThan: e2
	^ (comparator value: e1 value: e2) >= 0! !

!KSRMagnitudeFuzzySet methodsFor: 'element comparison' stamp: 'mg 6/30/2014 02:24'!
element: e1 equalOrLessThan: e2
	^ (comparator value: e1 value: e2) <= 0! !

!KSRMagnitudeFuzzySet methodsFor: 'element comparison' stamp: 'mg 6/30/2014 02:24'!
element: e1 greaterThan: e2
	^ (comparator value: e1 value: e2) > 0! !

!KSRMagnitudeFuzzySet methodsFor: 'element comparison' stamp: 'mg 6/30/2014 02:24'!
element: e1 lessThan: e2
	^ (comparator value: e1 value: e2) < 0! !

KSRMagnitudeFuzzySet subclass: #KSRNumericFuzzySet
	instanceVariableNames: 'enumerator interval'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KSRSummarizer'!

!KSRNumericFuzzySet class methodsFor: 'delta' stamp: 'mg 6/30/2014 01:56'!
defaultDelta
	^ 0.001! !

!KSRNumericFuzzySet class methodsFor: 'magnitude sets' stamp: 'mg 7/2/2014 00:42'!
equalOrGreaterThan: anObject
	^ self new
		membership: [:a| a >= anObject ifTrue: [1] ifFalse: [0]];
		specials: (Set with: anObject);
		name: 'equal or greater than ',anObject asString;
		interval: 
		(
			anObject - self defaultDelta -(self defaultDelta / 2) to:
			anObject + self defaultDelta + self defaultDelta by:
			self defaultDelta
		).		! !

!KSRNumericFuzzySet class methodsFor: 'magnitude sets' stamp: 'mg 7/2/2014 00:43'!
equalOrLessThan: anObject
	^ self new
		membership: [:a| a <= anObject ifTrue: [1] ifFalse: [0]];
		specials: (Set with: anObject);
		name: 'equal or less than ',anObject asString;
		interval: 
		(
			anObject - self defaultDelta -(self defaultDelta / 2) to:
			anObject + self defaultDelta + self defaultDelta by:
			self defaultDelta
		).			! !

!KSRNumericFuzzySet class methodsFor: 'magnitude sets' stamp: 'mg 7/2/2014 00:41'!
greaterThan: anObject
	^ self new
		membership: [:a| a > anObject ifTrue: [1] ifFalse: [0]];
		specials: (Set with: anObject);
		name: 'greater than ',anObject asString;
		interval: 
		(
			anObject - self defaultDelta -(self defaultDelta / 2) to:
			anObject + self defaultDelta + self defaultDelta by:
			self defaultDelta
		).		! !

!KSRNumericFuzzySet class methodsFor: 'magnitude sets' stamp: 'mg 7/2/2014 00:42'!
lessThan: anObject
	^ self new
		membership: [:a| a < anObject ifTrue: [1] ifFalse: [0]];
		specials: (Set with: anObject);
		name: 'less than ',anObject asString;
		interval: 
		(
			anObject - self defaultDelta - (self defaultDelta / 2) to:
			anObject + self defaultDelta + self defaultDelta by:
			self defaultDelta
		).		! !

!KSRNumericFuzzySet class methodsFor: 'numeric sets' stamp: 'mg 6/30/2014 01:56'!
loweringFrom: aNumber1 to: aNumber2
	^ self new
		membership: [:o|
			true caseOf:
			{
				[o <= aNumber1] -> [1].
				[o >= aNumber2] -> [0].
			}
			otherwise: [1 - ((o - aNumber1)/(aNumber2 - aNumber1))]
		];
		specials: (Set withAll: {aNumber1. aNumber2});
		interval: (aNumber1 to: aNumber2 by: self defaultDelta);
		name: 'somehow less than ',{aNumber1. aNumber2} average asString! !

!KSRNumericFuzzySet class methodsFor: 'numeric sets' stamp: 'mg 6/30/2014 01:57'!
raisingFrom: aNumber1 to: aNumber2
	^ self new
		membership: [:o|
			true caseOf:
			{
				[o <= aNumber1] -> [0].
				[o >= aNumber2] -> [1].
			}
			otherwise: [(o - aNumber1)/(aNumber2 - aNumber1)]
		];
		specials: (Set withAll: {aNumber1. aNumber2});
		interval: (aNumber1 to: aNumber2 by: self defaultDelta);
		name: 'somehow greater than ',{aNumber1. aNumber2} average asString			! !

!KSRNumericFuzzySet class methodsFor: 'simple sets' stamp: 'mg 6/30/2014 02:25'!
withElements: aCollection
	|set dictionary|
	set := Set new.
	dictionary := Dictionary new.
	aCollection do:
	[:o|
		set add: o key.
		dictionary at: o key put: o value
	].

	^ self new
		membership: [:a| dictionary at: a ifAbsent: [0]];
		specials: set;
		interval: (set min to: set max by: self defaultDelta);
		name: 'in ',aCollection printString! !

!KSRNumericFuzzySet methodsFor: 'operations' stamp: 'mg 6/30/2014 02:31'!
* aNumber
	^ super * aNumber
		enumerator: enumerator;
		interval: interval.! !

!KSRNumericFuzzySet methodsFor: 'operations' stamp: 'mg 6/30/2014 02:31'!
+ aNumber
	^ super + aNumber
		enumerator: enumerator;
		interval: interval.! !

!KSRNumericFuzzySet methodsFor: 'operations' stamp: 'mg 6/30/2014 02:31'!
- aNumber
	^ super - aNumber
		enumerator: enumerator;
		interval: interval.! !

!KSRNumericFuzzySet methodsFor: 'operations' stamp: 'mg 6/30/2014 02:31'!
/ aNumber
	^ super / aNumber
		enumerator: enumerator;
		interval: interval.! !

!KSRNumericFuzzySet methodsFor: 'operations' stamp: 'mg 7/2/2014 01:36'!
AND: aKSRFuzzySet norm: aBlock
	|newEnumerator newInterval|

	aKSRFuzzySet isNumericFuzzySet ifFalse: [^ super AND: aKSRFuzzySet norm: aBlock].	
	
	newInterval := 0 
					to: 
						(interval last - interval first) *
						(aKSRFuzzySet interval last - aKSRFuzzySet interval first)
					by: (interval increment min: aKSRFuzzySet interval increment).
					
	newEnumerator := [:n|
		(n \\ (interval last - interval first)) 
		@@
		(n // (interval last - interval first)).
	].	  
	
	^ (super AND: aKSRFuzzySet norm: aBlock)
		enumerator: newEnumerator;
		interval: newInterval.! !

!KSRNumericFuzzySet methodsFor: 'operations' stamp: 'mg 7/2/2014 01:38'!
OR: aKSRFuzzySet norm: aBlock
	|newEnumerator newInterval|

	aKSRFuzzySet isNumericFuzzySet ifFalse: [^ super OR: aKSRFuzzySet norm: aBlock].	
	
	newInterval := 0 
					to: 
						(interval last - interval first) *
						(aKSRFuzzySet interval last - aKSRFuzzySet interval first)
					by: (interval increment min: aKSRFuzzySet interval increment).
					
	newEnumerator := [:n|
		(n \\ (interval last - interval first)) 
		@@
		(n // (interval last - interval first)).
	].	  
	
	^ (super OR: aKSRFuzzySet norm: aBlock)
		enumerator: newEnumerator;
		interval: newInterval.! !

!KSRNumericFuzzySet methodsFor: 'operations' stamp: 'mg 6/30/2014 02:34'!
alphaCut: aNumber
	^ (super alphaCut: aNumber)
		enumerator: enumerator;
		interval: interval.
		! !

!KSRNumericFuzzySet methodsFor: 'operations' stamp: 'mg 6/30/2014 02:36'!
and: aKSRFuzzySet norm: aBlock
	^ (super and: aKSRFuzzySet norm: aBlock)
		enumerator: enumerator;
		interval: (
			(interval first min: aKSRFuzzySet interval first) to: 
			(interval last max: aKSRFuzzySet interval last) by:
			(interval increment min: aKSRFuzzySet interval increment)
		)! !

!KSRNumericFuzzySet methodsFor: 'operations' stamp: 'mg 7/1/2014 23:58'!
card
	"Działa tylko dla zbiorów:
		- jednowymiarowych,
		- gdy enumerator jest liniowy."
	constituents = self
	ifTrue: 
	[
		(
			self ? (enumerator value: interval first) = 0 and:
			[self ? (enumerator value: interval last) = 0]	
		) 
		ifFalse: [^ Float infinity].
		interval size <= 1 ifTrue: [^0].
		^ (interval collect: [:i | self ? (enumerator value: i)]) sum * 
		((enumerator value: interval second) - (enumerator value: interval first))
	]
	ifFalse: [^nil]
! !

!KSRNumericFuzzySet methodsFor: 'operations' stamp: 'mg 7/2/2014 00:33'!
cardBetween: aNumber1 and: aNumber2
	"Działa tylko dla zbiorów:
		- jednowymiarowych,
		- gdy enumerator jest liniowy."
	constituents = self
	ifTrue: 
	[
		^ (
			(aNumber1 to: aNumber2 by: self class defaultDelta) 
				collect: [:i | self ? (enumerator value: i)]
		) sum * self class defaultDelta
	]
	ifFalse: [^nil]
! !

!KSRNumericFuzzySet methodsFor: 'accessing' stamp: 'mg 6/29/2014 19:58'!
enumerator
	^ enumerator! !

!KSRNumericFuzzySet methodsFor: 'accessing' stamp: 'mg 6/29/2014 19:58'!
enumerator: aBlock
	enumerator := aBlock! !

!KSRNumericFuzzySet methodsFor: 'operations' stamp: 'mg 7/2/2014 01:15'!
fuzzinessForDomainBetween: aNumber1 and: aNumber2
	^ (self support cardBetween: aNumber1 and: aNumber2) / (aNumber2 - aNumber1)! !

!KSRNumericFuzzySet methodsFor: 'operations' stamp: 'mg 6/30/2014 02:42'!
height
	^ super height max: (interval collect: [:i | self ? (enumerator value: i)]) max! !

!KSRNumericFuzzySet methodsFor: 'initialize-release' stamp: 'mg 6/30/2014 02:27'!
initialize
	super initialize.
	interval := 0 to: 0 by: self class defaultDelta.
	enumerator := [:t|t].
	comparator := [:a :b| a - b].! !

!KSRNumericFuzzySet methodsFor: 'accessing' stamp: 'mg 6/29/2014 20:08'!
interval
	^ interval! !

!KSRNumericFuzzySet methodsFor: 'accessing' stamp: 'mg 6/29/2014 20:08'!
interval: anInterval
	interval := anInterval! !

!KSRNumericFuzzySet methodsFor: 'testing' stamp: 'mg 6/30/2014 10:35'!
isNumericFuzzySet
	^ true! !

!KSRNumericFuzzySet methodsFor: 'operations' stamp: 'mg 6/30/2014 02:36'!
not
	^ super not 
		enumerator: enumerator;
		interval: interval.! !

!KSRNumericFuzzySet methodsFor: 'operations' stamp: 'mg 6/30/2014 02:36'!
or: aKSRFuzzySet norm: aBlock
	^ (super or: aKSRFuzzySet norm: aBlock)
		enumerator: enumerator;
		interval: (
			(interval first min: aKSRFuzzySet interval first) to: 
			(interval last max: aKSRFuzzySet interval last) by:
			(interval increment min: aKSRFuzzySet interval increment)
		)! !

!KSRNumericFuzzySet methodsFor: 'plotting' stamp: 'mg 6/29/2014 20:22'!
plot2D
	^ self plot2DFrom: (enumerator value: interval first) 
	       to: (enumerator value: interval last)
	
	! !

!KSRNumericFuzzySet methodsFor: 'plotting' stamp: 'mg 6/29/2014 22:41'!
plot2DFrom: aNumber1 to: aNumber2
	MGPlotMorph new
		xRange: aNumber1 @ aNumber2;
		yRange: -0.1 @ 1.1;
		units: (aNumber2 - aNumber1 / 5) @ (1/10);
		yLabelsRight;
		addRelation: (
			MGRelationOnLeftToRightBlock new
				block: membership;
				continuous;
				color: Color blue
		);
		openInWindow
	
	! !

!KSRNumericFuzzySet methodsFor: 'plotting' stamp: 'mg 6/29/2014 20:38'!
plot3D
	self plot3DInRectangle: 
		((enumerator value: interval first) corner: (enumerator value: interval last))
	! !

!KSRNumericFuzzySet methodsFor: 'plotting' stamp: 'mg 6/29/2014 23:20'!
plot3DInRectangle: aRectangle
	MGPlotMorph new
		xRange: aRectangle left @ aRectangle right;
		yRange: aRectangle top @ aRectangle bottom;
		units: (aRectangle width / 10) @ (aRectangle height / 10);
		addRelation: (
			MGRelationOn2DBlock new
				block: [:x :y|membership value: x @@ y];
				continuous;
				color: Color blue
		);
		openInWindow
	
	! !

!KSRNumericFuzzySet methodsFor: 'operations' stamp: 'mg 6/30/2014 02:37'!
strictAlphaCut: aNumber
	^ (super strictAlphaCut: aNumber)
		enumerator: enumerator;
		interval: interval.! !

Object subclass: #KSRSummary
	instanceVariableNames: 'quantifier subjects qualifiers summarizer quality t1 t2'
	classVariableNames: 'Columns'
	poolDictionaries: ''
	category: 'KSRSummarizer'!

KSRSummary subclass: #KSRAbsoluteSummary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KSRSummarizer'!

!KSRAbsoluteSummary class methodsFor: 'quantifiers' stamp: 'mg 6/30/2014 03:58'!
some
	^ (KSRNumericFuzzySet greaterThan: 0) name: 'some'! !

KSRAbsoluteSummary subclass: #KSRYagerAbsoluteSummary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KSRSummarizer'!

!KSRYagerAbsoluteSummary class methodsFor: 'generating' stamp: 'mg 6/30/2014 13:14'!
random
	|quantifier subject summarizer qualifier|
	quantifier := self perform: (self quantifiers atRandom).
	quantifier := {
		quantifier. 
		quantifier not name: 'it is not true that ',quantifier name
	} atRandom.
	subject := self people.
	summarizer := self randomCombinationOf: self summarizers.
	qualifier := self randomCombinationOf: self summarizers.
	^ self new 
		quantifier: quantifier;
		addSubject: subject qualified: qualifier;
		summarizer: summarizer! !

!KSRYagerAbsoluteSummary methodsFor: 'summarizing' stamp: 'mg 7/2/2014 01:25'!
calculateT1ForDatabase: database
	|r|
	
	r :=	database inject: 0 into: [:total :row|
		total + ((subjects first /\ summarizer /\ qualifiers first) ? row)
	].

	Transcript showln: 'r=',r asString.
	
	^ quantifier ? r! !

KSRSummary subclass: #KSRRelativeSummary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KSRSummarizer'!

KSRRelativeSummary subclass: #KSRFirstMultiSubjectSummary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KSRSummarizer'!

!KSRFirstMultiSubjectSummary class methodsFor: 'generating' stamp: 'mg 6/30/2014 15:31'!
random
	|quantifier subject1 subject2 summarizer|
	quantifier := self perform: (self quantifiers atRandom).
	quantifier := {
		quantifier. 
		quantifier not name: 'it is not true that ',quantifier name
	} atRandom.
	subject1 := self perform: (self subjects atRandom).
	subject2 := self perform: (self subjects atRandom).	
	summarizer := self randomCombinationOf: self summarizers.
	^ self new 
		quantifier: quantifier;
		addSubject: subject1 qualified: subject1;
		addSubject: subject2 qualified: subject1;		
		summarizer: summarizer! !

!KSRFirstMultiSubjectSummary methodsFor: 'summarizing' stamp: 'mg 7/2/2014 01:26'!
calculateT1ForDatabase: database
	|numerator1 denominator1 numerator2 denominator2 r|
	
	numerator1 := database inject: 0 into: [:total :row|
		total + ((subjects first /\ summarizer) ? row)
	].

	denominator1 := database inject: 0 into: [:total :row|
		total + ((subjects first) ? row)
	].

	numerator2 := database inject: 0 into: [:total :row|
		total + ((subjects second /\ summarizer) ? row)
	].

	denominator2 := database inject: 0 into: [:total :row|
		total + ((subjects second) ? row)
	].

	(denominator1 = 0 or: [denominator2 = 0] or: [numerator1 = 0 and: [numerator2 = 0]]) ifTrue: [^0].

	r := (numerator1 / denominator1) 
	     / 
		 (
		 	(numerator1 / denominator1) 
			+ 
			(numerator2 / denominator2)
		 ).
	
	^ quantifier ? r! !

!KSRFirstMultiSubjectSummary methodsFor: 'printing' stamp: 'mg 6/30/2014 15:22'!
printOn: aStream
	aStream print: quantifier.
	aStream nextPut: $ .
	aStream print: subjects first.
	aStream nextPutAll: ' relatively to '.
	aStream print: subjects second.
	aStream nextPutAll: ' are '.
	aStream print: summarizer.! !

KSRRelativeSummary subclass: #KSRMoreSummary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KSRSummarizer'!

!KSRMoreSummary class methodsFor: 'generating' stamp: 'mg 6/30/2014 15:18'!
random
	|subject1 subject2 summarizer|
	subject1 := self perform: (self subjects atRandom).
	subject2 := self perform: (self subjects atRandom).	
	summarizer := self randomCombinationOf: self summarizers.
	^ self new 
		addSubject: subject1 qualified: subject1;
		addSubject: subject2 qualified: subject1;		
		summarizer: summarizer! !

!KSRMoreSummary methodsFor: 'summarizing' stamp: 'mg 7/2/2014 01:26'!
calculateT1ForDatabase: database
	|value1 value2 r|
	
	value1 := database inject: 0 into: [:total :row|
		total + ((subjects first /\ summarizer) ? row)
	].


	value2 := database inject: 0 into: [:total :row|
		total + ((subjects second /\ summarizer) ? row)
	].

	(value1 = 0 and: [value2 = 0]) ifTrue: [^0].

	r := value1 / (value1 + value2).
	
	^ r! !

!KSRMoreSummary methodsFor: 'printing' stamp: 'mg 6/30/2014 15:27'!
printOn: aStream
	aStream nextPutAll: 'More '.
	aStream print: subjects first.
	aStream nextPutAll: ' than '.
	aStream print: subjects second.
	aStream nextPutAll: ' are '.
	aStream print: summarizer.! !

!KSRRelativeSummary class methodsFor: 'quantifiers' stamp: 'mg 6/30/2014 04:14'!
aboutHalfOf
	^ (KSRNumericFuzzySet raisingFrom: 0.4 to: 0.5) /\
	  (KSRNumericFuzzySet loweringFrom: 0.5 to: 0.6) name: 'about half of'! !

!KSRRelativeSummary class methodsFor: 'quantifiers' stamp: 'mg 6/30/2014 04:12'!
all
	^ (KSRNumericFuzzySet equalOrGreaterThan: 1) name: 'all'! !

!KSRRelativeSummary class methodsFor: 'quantifiers' stamp: 'mg 6/30/2014 04:12'!
almostAll
	^ (KSRNumericFuzzySet raisingFrom: 0.85 to: 0.9) /\ (KSRNumericFuzzySet loweringFrom: 0.95 to: 1) name: 'almost all'! !

!KSRRelativeSummary class methodsFor: 'quantifiers' stamp: 'mg 6/30/2014 11:00'!
few
	^ (KSRNumericFuzzySet loweringFrom: 0.2 to: 0.4) name: 'few'! !

!KSRRelativeSummary class methodsFor: 'quantifiers' stamp: 'mg 6/30/2014 04:03'!
many
	^ (KSRNumericFuzzySet raisingFrom: 0.3 to: 0.8) name: 'many'! !

!KSRRelativeSummary class methodsFor: 'quantifiers' stamp: 'mg 6/30/2014 03:56'!
most
	^ (KSRNumericFuzzySet raisingFrom: 0.5 to: 0.7) name: 'most'! !

KSRRelativeSummary subclass: #KSRSecondMultiSubjectSummary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KSRSummarizer'!

!KSRSecondMultiSubjectSummary class methodsFor: 'generating' stamp: 'mg 6/30/2014 15:17'!
random
	|quantifier subject1 subject2 summarizer qualifier|
	quantifier := self perform: (self quantifiers atRandom).
	quantifier := {
		quantifier. 
		quantifier not name: 'it is not true that ',quantifier name
	} atRandom.
	subject1 := self perform: (self subjects atRandom).
	subject2 := self perform: (self subjects atRandom).	
	summarizer := self randomCombinationOf: self summarizers.
	qualifier := self randomCombinationOf: self summarizers.
	^ self new 
		quantifier: quantifier;
		addSubject: subject1 qualified: qualifier;
		addSubject: subject2 qualified: subject2;		
		summarizer: summarizer! !

!KSRSecondMultiSubjectSummary methodsFor: 'summarizing' stamp: 'mg 7/2/2014 01:26'!
calculateT1ForDatabase: database
	|numerator1 denominator1 numerator2 denominator2 r|
	
	numerator1 := database inject: 0 into: [:total :row|
		total + ((subjects first /\ summarizer /\ qualifiers first) ? row)
	].

	denominator1 := database inject: 0 into: [:total :row|
		total + ((subjects first) ? row)
	].

	numerator2 := database inject: 0 into: [:total :row|
		total + ((subjects second /\ summarizer /\ qualifiers first) ? row)
	].

	denominator2 := database inject: 0 into: [:total :row|
		total + ((subjects second) ? row)
	].

	(denominator1 = 0 or: [denominator2 = 0] or: [numerator1 = 0 and: [numerator2 = 0]]) ifTrue: [^0].

	r := (numerator1 / denominator1) 
	     / 
		 (
		 	(numerator1 / denominator1) 
			+ 
			(numerator2 / denominator2)
		 ).
	
	^ quantifier ? r! !

!KSRSecondMultiSubjectSummary methodsFor: 'printing' stamp: 'mg 6/30/2014 15:23'!
printOn: aStream
	aStream print: quantifier.
	aStream nextPut: $ .
	aStream print: subjects first.
	aStream nextPutAll: ' relatively to '.
	aStream print: subjects second.
	aStream nextPutAll: ' who are '.
	aStream print: qualifiers first.
	aStream nextPutAll: ' are '.
	aStream print: summarizer.! !

KSRRelativeSummary subclass: #KSRThirdMultiSubjectSummary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KSRSummarizer'!

!KSRThirdMultiSubjectSummary class methodsFor: 'generating' stamp: 'mg 6/30/2014 15:17'!
random
	|quantifier subject1 subject2 summarizer qualifier|
	quantifier := self perform: (self quantifiers atRandom).
	quantifier := {
		quantifier. 
		quantifier not name: 'it is not true that ',quantifier name
	} atRandom.
	subject1 := self perform: (self subjects atRandom).
	subject2 := self perform: (self subjects atRandom).	
	summarizer := self randomCombinationOf: self summarizers.
	qualifier := self randomCombinationOf: self summarizers.
	^ self new 
		quantifier: quantifier;
		addSubject: subject1 qualified: qualifier;
		addSubject: subject2 qualified: subject2;		
		summarizer: summarizer! !

!KSRThirdMultiSubjectSummary methodsFor: 'summarizing' stamp: 'mg 7/2/2014 01:26'!
calculateT1ForDatabase: database
	|numerator1 denominator1 numerator2 denominator2 numerator3 denominator3 r|
	
	numerator1 := database inject: 0 into: [:total :row|
		total + ((subjects first /\ summarizer /\ qualifiers first) ? row)
	].

	denominator1 := database inject: 0 into: [:total :row|
		total + ((subjects first) ? row)
	].

	numerator2 := database inject: 0 into: [:total :row|
		total + ((subjects first /\ summarizer) ? row)
	].

	denominator2 := denominator1.

	numerator3 := database inject: 0 into: [:total :row|
		total + ((subjects second /\ summarizer) ? row)
	].

	denominator3 := database inject: 0 into: [:total :row|
		total + ((subjects second) ? row)
	].

	(denominator1 = 0 or: [denominator2 = 0] or: [denominator3 = 0] or: [numerator1 = 0 and: [numerator2 = 0] and: [numerator3 = 0]]) ifTrue: [^0].

	r := (numerator1 / denominator1) 
	     / 
		 (
		 	(numerator2 / denominator2) 
			+ 
			(numerator3 / denominator3)
		 ).
	
	^ quantifier ? r! !

!KSRThirdMultiSubjectSummary methodsFor: 'printing' stamp: 'mg 6/30/2014 15:24'!
printOn: aStream
	aStream print: quantifier.
	aStream nextPut: $ .
	aStream print: subjects first.
	aStream nextPutAll: ' who are '.
	aStream print: qualifiers first.	
	aStream nextPutAll: ' relatively to '.
	aStream print: subjects second.
	aStream nextPutAll: ' are '.
	aStream print: summarizer.! !

KSRRelativeSummary subclass: #KSRYagerRelativeSummary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KSRSummarizer'!

!KSRYagerRelativeSummary class methodsFor: 'generating' stamp: 'mg 6/30/2014 13:15'!
random
	|quantifier subject summarizer qualifier|
	quantifier := self perform: (self quantifiers atRandom).
	quantifier := {
		quantifier. 
		quantifier not name: 'it is not true that ',quantifier name
	} atRandom.
	subject := self people.
	summarizer := self randomCombinationOf: self summarizers.
	qualifier := self randomCombinationOf: self summarizers.
	^ self new 
		quantifier: quantifier;
		addSubject: subject qualified: qualifier;
		summarizer: summarizer! !

!KSRYagerRelativeSummary methodsFor: 'summarizing' stamp: 'mg 7/2/2014 01:26'!
calculateT1ForDatabase: database
	|numerator denominator|
	
	numerator := database inject: 0 into: [:total :row|
		total + ((subjects first /\ summarizer /\ qualifiers first) ? row)
	].

	denominator := database inject: 0 into: [:total :row|
		total + ((subjects first /\ qualifiers first) ? row)
	].

	denominator = 0 ifTrue: [^0].

	Transcript showln: 'r=',numerator asString,'/',denominator asString.
	
	^ quantifier ? (numerator / denominator)! !

!KSRSummary class methodsFor: 'general summarizers' stamp: 'mg 6/30/2014 13:03'!
absent
	^ KSRFuzzySet present name: 'absent'! !

!KSRSummary class methodsFor: 'subjects' stamp: 'mg 6/30/2014 09:38'!
admins
	^ (
		KSRFuzzySet present ><
		(KSRFuzzySet withElements: {#'admin.'\1}) ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present
	) name: 'admins'! !

!KSRSummary class methodsFor: 'age summarizers' stamp: 'mg 6/30/2014 13:02'!
adults
	^ (KSRNumericFuzzySet raisingFrom: 16 to: 21) 
	  name: 'adults'! !

!KSRSummary class methodsFor: 'subjects' stamp: 'mg 6/30/2014 09:38'!
blueCollars
	^ (
		KSRFuzzySet present ><
		(KSRFuzzySet withElements: {#'blue-collar'\1}) ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present
	) name: 'blue collars'! !

!KSRSummary class methodsFor: 'age summarizers' stamp: 'mg 6/30/2014 13:01'!
children
	^ (KSRNumericFuzzySet loweringFrom: 9 to: 11) name: 'children'! !

!KSRSummary class methodsFor: 'marital summarizers' stamp: 'mg 6/30/2014 16:52'!
divorced
	^(KSRFuzzySet withElements: {#divorced\1.0}) name: 'divorced or widowed'! !

!KSRSummary class methodsFor: 'job summarizers' stamp: 'mg 6/30/2014 16:50'!
employed
	^(
		KSRFuzzySet withElements: {
			#'admin.'\1.
			#'blue-collar'\1.
			#'entrepreneur'\1.
			#'housemaid'\1.
			#'management'\1.
			#'retired'\0.
			#'self-employed'\1.
			#'services'\1.
			#'student'\0.1.
			#'technician'\1.
			#'unemployed'\0.
			#'unknown'\0.5
		}
	) name: 'employed'.! !

!KSRSummary class methodsFor: 'subjects' stamp: 'mg 6/30/2014 09:38'!
entrepreneurs
	^ (
		KSRFuzzySet present ><
		(KSRFuzzySet withElements: {#'entrepreneur'\1}) ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present
	) name: 'entrepreneurs'! !

!KSRSummary class methodsFor: 'generating' stamp: 'mg 6/30/2014 12:04'!
generate
	self subclassResponsibility! !

!KSRSummary class methodsFor: 'job summarizers' stamp: 'mg 6/30/2014 16:50'!
havingHighSocialStatus
	^(
		KSRFuzzySet withElements: {
			#'admin.'\0.8.
			#'blue-collar'\0.
			#'entrepreneur'\1.
			#'housemaid'\0.
			#'management'\0.9.
			#'retired'\0.3.
			#'self-employed'\1.
			#'services'\0.3.
			#'student'\0.
			#'technician'\0.
			#'unemployed'\0.
			#'unknown'\0.5
		}
	) name: 'having high social status'.! !

!KSRSummary class methodsFor: 'job summarizers' stamp: 'mg 6/30/2014 16:49'!
havingLowSocialStatus
	^(
		KSRFuzzySet withElements: {
			#'admin.'\0.
			#'blue-collar'\0.
			#'entrepreneur'\0.
			#'housemaid'\0.9.
			#'management'\0.
			#'retired'\0.3.
			#'self-employed'\0.
			#'services'\0.
			#'student'\0.9.
			#'technician'\0.4.
			#'unemployed'\1.
			#'unknown'\0.5
		}
	) name: 'having low social status'.! !

!KSRSummary class methodsFor: 'subjects' stamp: 'mg 6/30/2014 09:38'!
housemaids
	^ (
		KSRFuzzySet present ><
		(KSRFuzzySet withElements: {#'housemaid'\1}) ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present
	) name: 'housemaids'! !

!KSRSummary class methodsFor: 'class initialization' stamp: 'mg 7/2/2014 03:04'!
initialize
	"KSRSummary initialize."
	Columns := {
		'age'.
		'job'.
		'marital'.
		'education'.
		'default'.
		'housing'.
		'loan'.
		'contact'.
		'month'.
		'day of week'.
		'duration'.
		'campaign'.
		'pdays'.
		'previous'.
		'poutcome'.
		'emp.var.rate'.
		'cons.price.idx'.		
		'cons.conf.idx'.
		'euribor3m'.
		'nr.employed'.
		'deposit'
	}! !

!KSRSummary class methodsFor: 'subjects' stamp: 'mg 6/30/2014 09:38'!
managers
	^ (
		KSRFuzzySet present ><
		(KSRFuzzySet withElements: {#'management'\1}) ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present
	) name: 'managers'! !

!KSRSummary class methodsFor: 'marital summarizers' stamp: 'mg 6/30/2014 16:51'!
married
	^(KSRFuzzySet withElements: {#married\1.0}) name: 'married'! !

!KSRSummary class methodsFor: 'generating' stamp: 'mg 6/30/2014 12:02'!
maximalNumberOfSummarizers
	^ 4! !

!KSRSummary class methodsFor: 'marital summarizers' stamp: 'mg 6/30/2014 16:52'!
notKnownWhetherMarried
	^(KSRFuzzySet withElements: {#unknown\1.0}) name: 'not known whether married'! !

!KSRSummary class methodsFor: 'age summarizers' stamp: 'mg 6/30/2014 13:02'!
old
	^ (KSRNumericFuzzySet raisingFrom: 50 to: 65) 
	  name: 'old'! !

!KSRSummary class methodsFor: 'general subjects' stamp: 'mg 6/30/2014 10:01'!
people
	^ (
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present
	) name: 'people'! !

!KSRSummary class methodsFor: 'subjects' stamp: 'mg 6/30/2014 17:10'!
peopleWithUnknownJob
	^ (
		KSRFuzzySet present ><
		(KSRFuzzySet withElements: {#'unknown'\1}) ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present
	) name: 'people with unknown job'! !

!KSRSummary class methodsFor: 'general summarizers' stamp: 'mg 6/30/2014 13:03'!
present
	^ KSRFuzzySet present name: 'present'! !

!KSRSummary class methodsFor: 'generating' stamp: 'mg 6/30/2014 11:27'!
quantifiers
	^ self class allMethodsInCategory: 'quantifiers'! !

!KSRSummary class methodsFor: 'generating' stamp: 'mg 6/30/2014 12:16'!
random
	self subclassResponsibility! !

!KSRSummary class methodsFor: 'generating' stamp: 'mg 6/30/2014 13:04'!
randomCombinationOf: anOrderedCollection
	|first count indexes collection combination|
	count := (0 to: self maximalNumberOfSummarizers) atRandom.
	indexes := ((1 to: count) collect: [:i|(1 to: anOrderedCollection size) atRandom]) asSet.
	
	collection := anOrderedCollection withIndexCollect: [:elements :index|
		((indexes includes: index) not or: [elements isEmpty])
		ifFalse: [
			|fset|
			fset := self perform: (elements atRandom).
			{fset. fset not} atRandom
		]
		ifTrue: [self present]
	].

	first := true.

	collection do: [:element|
		first
		ifTrue:
		[
			first := false.
			combination := element.
		]
		ifFalse:
		[
			combination := combination >< element
		]
	].
	
	^combination
	
	! !

!KSRSummary class methodsFor: 'subjects' stamp: 'mg 6/30/2014 09:38'!
retired
	^ (
		KSRFuzzySet present ><
		(KSRFuzzySet withElements: {#'retired'\1}) ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present
	) name: 'retired'! !

!KSRSummary class methodsFor: 'subjects' stamp: 'mg 6/30/2014 09:38'!
selfEmployed
	^ (
		KSRFuzzySet present ><
		(KSRFuzzySet withElements: {#'self-employed'\1}) ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present
	) name: 'self employed'! !

!KSRSummary class methodsFor: 'marital summarizers' stamp: 'mg 6/30/2014 16:51'!
single
	^(KSRFuzzySet withElements: {#single\1.0}) name: 'single'! !

!KSRSummary class methodsFor: 'subjects' stamp: 'mg 6/30/2014 09:38'!
students
	^ (
		KSRFuzzySet present ><
		(KSRFuzzySet withElements: {#'student'\1}) ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present
	) name: 'students'! !

!KSRSummary class methodsFor: 'generating' stamp: 'mg 6/30/2014 11:27'!
subjects
	^ self class allMethodsInCategory: 'subjects'! !

!KSRSummary class methodsFor: 'generating' stamp: 'mg 7/2/2014 03:04'!
summarizers
	|collection|
	collection := OrderedCollection new.
	Columns do: [:column|
		|category|
		category := column,' summarizers'.
		(self class organization categories includes: category)
			ifFalse: [self class organization addCategory: category].		
		collection add: (self class allMethodsInCategory: category)		
	].

	^ collection! !

!KSRSummary class methodsFor: 'subjects' stamp: 'mg 6/30/2014 09:38'!
technicians
	^ (
		KSRFuzzySet present ><
		(KSRFuzzySet withElements: {#'technician'\1}) ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present
	) name: 'technicians'! !

!KSRSummary class methodsFor: 'age summarizers' stamp: 'mg 6/30/2014 13:01'!
teenagers
	^ (KSRNumericFuzzySet raisingFrom: 10 to: 12 ) 
	  /\ 
	  (KSRNumericFuzzySet loweringFrom: 18 to: 20) 
	  name: 'teenagers'! !

!KSRSummary class methodsFor: 'education summarizers' stamp: 'mg 6/30/2014 16:58'!
uneducated
	^ (
		KSRFuzzySet withElements:
		{
			#'basic.4y'\0.6.
			#'basic.6y'\0.4.
			#'basic.9y'\0.2.
			#'high.school'\0.1.
			#'illiterate'\1.
			#'professional.course'\0.1.
			#'university.degree'\0.
			#'unknown'\0.5
		}
	) name: 'uneducated'! !

!KSRSummary class methodsFor: 'subjects' stamp: 'mg 6/30/2014 09:38'!
unemployed
	^ (
		KSRFuzzySet present ><
		(KSRFuzzySet withElements: {#'unemployed'\1}) ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present
	) name: 'unemployed'! !

!KSRSummary class methodsFor: 'marital summarizers' stamp: 'mg 6/30/2014 16:51'!
unknown
	^(KSRFuzzySet withElements: {#unknown\1.0}) name: 'unknown'! !

!KSRSummary class methodsFor: 'education summarizers' stamp: 'mg 6/30/2014 16:57'!
veryEducated
	^ (
		KSRFuzzySet withElements:
		{
			#'basic.4y'\0.05.
			#'basic.6y'\0.1.
			#'basic.9y'\0.2.
			#'high.school'\0.5.
			#'illiterate'\0.
			#'professional.course'\0.5.
			#'university.degree'\1.
			#'unknown'\0.3
		}
	) name: 'very educated'! !

!KSRSummary class methodsFor: 'subjects' stamp: 'mg 6/30/2014 17:10'!
workingInServices
	^ (
		KSRFuzzySet present ><
		(KSRFuzzySet withElements: {#'services'\1}) ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present ><
		KSRFuzzySet present
	) name: 'working in services'! !

!KSRSummary class methodsFor: 'age summarizers' stamp: 'mg 6/30/2014 13:01'!
youngAdults
	^ (KSRNumericFuzzySet raisingFrom: 18 to: 21) 
	  /\ 
	  (KSRNumericFuzzySet loweringFrom: 28 to: 31) 
	  name: 'young adults'! !

!KSRSummary methodsFor: 'accessing' stamp: 'mg 6/30/2014 00:42'!
addSubject: subject qualified: qualifier
	subjects add: subject.
	qualifiers add: qualifier! !

!KSRSummary methodsFor: 'summarizing' stamp: 'mg 7/2/2014 01:22'!
calculateT1ForDatabase: database
	self subclassResponsibility! !

!KSRSummary methodsFor: 'summarizing' stamp: 'mg 7/2/2014 02:07'!
calculateT2ForDatabase: database values: values minimums: minimums maximums: maximums
	^ ((
		summarizer constituentsCollection withIndexCollect: [:summarizerConstituent :index|
			summarizerConstituent isNumericFuzzySet 
			ifTrue: 
			[
				summarizerConstituent 
					fuzzinessForDomainBetween: (minimums at: index) 
					and: (maximums at: index)
			]
			ifFalse: 
			[
				summarizerConstituent specials 
				ifEmpty:[nil]
				ifNotEmpty:
				[
					summarizerConstituent
						fuzzinessForDomainWithCardinality: (values at: index) size
				]
			]
		]
	) reject: [:o| o isNil]) 
	ifEmpty: [^ 0]
	ifNotEmpty:[:f|
		1 - ((f reduce: [:a :b| a * b]) nthRoot: f size)
	]! !

!KSRSummary methodsFor: 'description' stamp: 'mg 6/30/2014 13:19'!
description
	^ (((self fullPrintString 
		copyReplaceAll: ' AND present' with: '')
		copyReplaceAll: 'present AND ' with: '')
		copyReplaceAll: 'AND' with: 'and')
		copyReplaceAll: ' who are present' with: ''.! !

!KSRSummary methodsFor: 'initialize-release' stamp: 'mg 6/30/2014 00:39'!
initialize
	super initialize.
	subjects := OrderedCollection new.
	qualifiers := OrderedCollection new.! !

!KSRSummary methodsFor: 'printing' stamp: 'mg 6/30/2014 15:19'!
printOn: aStream
	aStream print: quantifier.
	aStream nextPut: $ .
	subjects with: qualifiers do: [:subject :qualifier|
		aStream print: subject.
		aStream nextPutAll: ' who are '.
		aStream print: qualifier.
		aStream nextPutAll: ' '
	].
	aStream nextPutAll: ' are '.
	aStream print: summarizer.! !

!KSRSummary methodsFor: 'summarizing' stamp: 'mg 7/2/2014 00:54'!
quality
	^ quality! !

!KSRSummary methodsFor: 'accessing' stamp: 'mg 6/30/2014 10:27'!
quantifier
	^ quantifier! !

!KSRSummary methodsFor: 'accessing' stamp: 'mg 6/30/2014 10:27'!
quantifier: aKSRNumericFuzzySet
	quantifier := aKSRNumericFuzzySet! !

!KSRSummary methodsFor: 'accessing' stamp: 'mg 6/30/2014 00:42'!
removeAllSubjectsAndQualifiers
	subjects removeAll.
	qualifiers removeAll! !

!KSRSummary methodsFor: 'summarizing' stamp: 'mg 7/2/2014 01:31'!
summarize: database values: values minimums: minimums maximums: maximums
	t1 := self calculateT1ForDatabase: database.
	t2 := self 
		calculateT2ForDatabase: database 
		values: values 
		minimums: minimums 
		maximums: maximums.
	quality := t1.
	^ quality! !

!KSRSummary methodsFor: 'accessing' stamp: 'mg 6/30/2014 00:37'!
summarizer

	^ summarizer! !

!KSRSummary methodsFor: 'accessing' stamp: 'mg 6/30/2014 00:37'!
summarizer: anObject

	summarizer := anObject! !

!KSRSummary methodsFor: 'summarizing' stamp: 'mg 7/2/2014 00:54'!
t1
	^ t1! !

!KSRSummary methodsFor: 'summarizing' stamp: 'mg 7/2/2014 00:56'!
t2
	^ t2! !

!Object methodsFor: '*ksrsummarizer' stamp: 'mg 6/29/2014 20:32'!
@@ anObject
	^ KSRPoint x: self y: anObject! !

!Object methodsFor: '*ksrsummarizer' stamp: 'mg 6/29/2014 20:34'!
\ aNumber
	^ KSRFuzzySetElement key: self value: aNumber! !

Association subclass: #KSRFuzzySetElement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KSRSummarizer'!

!KSRFuzzySetElement methodsFor: 'printing' stamp: 'mg 6/29/2014 20:34'!
printOn: aStream
	aStream print: key.
	aStream nextPutAll: ' \ '.
	aStream print: value.! !
KSRSummary initialize!
